p a c k a g e   P O E : : C o m p o n e n t : : D B I x : : M y S e r v e r : : C l i e n t ; 
 
 #   S t a n d a r d   s t u f f   t o   c a t c h   e r r o r s 
 u s e   s t r i c t   q w ( s u b s   v a r s   r e f s ) ; 	 	 	 	 #   M a k e   s u r e   w e   c a n ' t   m e s s   u p 
 u s e   w a r n i n g s   F A T A L   = >   ' a l l ' ; 	 	 	 	 #   E n a b l e   p r i n t i n g s   t o   c a t c h   e r r o r s 
 
 u s e   v a r s   q w ( $ V E R S I O N   @ I S A ) ; 
 
 #   I n i t i a l i z e   o u r   v e r s i o n 
 $ V E R S I O N   =   ' 0 . 0 1 _ 0 1 ' ; 
 
 u s e   P O E ; 
 u s e   P O E : : K e r n e l ; 
 u s e   C a r p   q w (   c r o a k   ) ; 
 u s e   C l a s s : : A c c e s s o r : : F a s t ; 
 u s e   b a s e   ' C l a s s : : A c c e s s o r : : F a s t ' ; 
 
 _ _ P A C K A G E _ _ - > m k _ a c c e s s o r s ( q w / 
         u s e r n a m e 
         p a s s w o r d 
         w h e e l 
         s e s s i o n _ i d 
         s e s s i o n 
         b a n n e r 
         s a l t 
         c h a r s e t 
         t i d 
         p a c k e t _ c o u n t 
         s c r a m b l e 
         u s e r n a m e 
         a u t h e n t i c a t e d 
         q u e r y _ h a n d l e r s 
         r o w s _ b u f f e r 
         d a t a b a s e 
         s e r v e r _ c l a s s 
         w a n t _ d b _ n a m e 
 / ) ; 
 
 u s e   D a t a : : D u m p e r ; 
 
 s u b   D E B U G   { 1 } 
 B E G I N   { 
 	 #   D e b u g   f u n ! 
 	 i f   (   !   d e f i n e d   & D E B U G   )   { 
 	 	 e v a l   " s u b   D E B U G   ( )   {   0   } " ; 
 	 } 
 	 #   O u r   o w n   d e f i n i t i o n   o f   t h e   m a x   r e t r i e s 
 	 i f   (   !   d e f i n e d   & M A X _ R E T R I E S   )   { 
 	 	 e v a l   " s u b   M A X _ R E T R I E S   ( )   {   5   } " ; 
 	 } 
 } 
 
 
 u s e   c o n s t a n t   M Y S E R V E R _ P A C K E T _ C O U N T 	 = >   0 ; 
 u s e   c o n s t a n t   M Y S E R V E R _ S O C K E T 	 	 = >   1 ; 
 u s e   c o n s t a n t   M Y S E R V E R _ D A T A B A S E 	 	 = >   4 ; 
 u s e   c o n s t a n t   M Y S E R V E R _ T H R E A D _ I D 	 	 = >   5 ; 
 u s e   c o n s t a n t   M Y S E R V E R _ S C R A M B L E 	 	 = >   6 ; 
 u s e   c o n s t a n t   M Y S E R V E R _ D B H 	 	 = >   7 ; 
 u s e   c o n s t a n t   M Y S E R V E R _ P A R S E R 	 	 = >   8 ; 
 u s e   c o n s t a n t   M Y S E R V E R _ B A N N E R 	 	 = >   9 ; 
 u s e   c o n s t a n t   M Y S E R V E R _ S E R V E R _ C H A R S E T 	 = >   1 0 ; 
 u s e   c o n s t a n t   M Y S E R V E R _ C L I E N T _ C H A R S E T 	 = >   1 1 ; 
 u s e   c o n s t a n t   M Y S E R V E R _ S A L T 	 	 = >   1 2 ; 
 
 u s e   c o n s t a n t   F I E L D _ C A T A L O G 	 	 = >   0 ; 
 u s e   c o n s t a n t   F I E L D _ D B 	 	 	 = >   1 ; 
 u s e   c o n s t a n t   F I E L D _ T A B L E 	 	 = >   2 ; 
 u s e   c o n s t a n t   F I E L D _ O R G _ T A B L E 	 	 = >   3 ; 
 u s e   c o n s t a n t   F I E L D _ N A M E 	 	 	 = >   4 ; 
 u s e   c o n s t a n t   F I E L D _ O R G _ N A M E 	 	 = >   5 ; 
 u s e   c o n s t a n t   F I E L D _ L E N G T H 	 	 = >   6 ; 
 u s e   c o n s t a n t   F I E L D _ T Y P E 	 	 	 = >   7 ; 
 u s e   c o n s t a n t   F I E L D _ F L A G S 	 	 = >   8 ; 
 u s e   c o n s t a n t   F I E L D _ D E C I M A L S 	 	 = >   9 ; 
 u s e   c o n s t a n t   F I E L D _ D E F A U L T 	 	 = >   1 0 ; 
 
 
 # 
 #   T h i s   c o m e s   f r o m   i n c l u d e / m y s q l _ c o m . h   o f   t h e   M y S Q L   s o u r c e 
 # 
 
 u s e   c o n s t a n t   C L I E N T _ L O N G _ P A S S W O R D 	 = >   1 ; 
 u s e   c o n s t a n t   C L I E N T _ F O U N D _ R O W S 	 	 = >   2 ; 
 u s e   c o n s t a n t   C L I E N T _ L O N G _ F L A G 	 	 = >   4 ; 
 u s e   c o n s t a n t   C L I E N T _ C O N N E C T _ W I T H _ D B 	 = >   8 ; 
 u s e   c o n s t a n t   C L I E N T _ N O _ S C H E M A 	 	 = >   1 6 ; 
 u s e   c o n s t a n t   C L I E N T _ C O M P R E S S 	 	 = >   3 2 ; 	 	 #   M u s t   i m p l e m e n t   t h a t   o n e 
 u s e   c o n s t a n t   C L I E N T _ O D B C 	 	 = >   6 4 ; 
 u s e   c o n s t a n t   C L I E N T _ L O C A L _ F I L E S 	 	 = >   1 2 8 ; 
 u s e   c o n s t a n t   C L I E N T _ I G N O R E _ S P A C E 	 = >   2 5 6 ; 
 u s e   c o n s t a n t   C L I E N T _ P R O T O C O L _ 4 1 	 	 = >   5 1 2 ; 
 u s e   c o n s t a n t   C L I E N T _ I N T E R A C T I V E 	 	 = >   1 0 2 4 ; 
 u s e   c o n s t a n t   C L I E N T _ S S L 	 	 	 = >   2 0 4 8 ; 	 #   M u s t   i m p l e m e n t   t h a t   o n e 
 u s e   c o n s t a n t   C L I E N T _ I G N O R E _ S I G P I P E 	 = >   4 0 9 6 ; 
 u s e   c o n s t a n t   C L I E N T _ T R A N S A C T I O N S 	 = >   8 1 9 2 ; 
 u s e   c o n s t a n t   C L I E N T _ R E S E R V E D   	 	 = >   1 6 3 8 4 ; 
 u s e   c o n s t a n t   C L I E N T _ S E C U R E _ C O N N E C T I O N 	 = >   3 2 7 6 8 ; 
 u s e   c o n s t a n t   C L I E N T _ M U L T I _ S T A T E M E N T S 	 = >   1   < <   1 6 ; 
 u s e   c o n s t a n t   C L I E N T _ M U L T I _ R E S U L T S 	 = >   1   < <   1 7 ; 
 u s e   c o n s t a n t   C L I E N T _ S S L _ V E R I F Y _ S E R V E R _ C E R T 	 = >   1   < <   3 0 ; 
 u s e   c o n s t a n t   C L I E N T _ R E M E M B E R _ O P T I O N S 	 	 = >   1   < <   3 1 ; 
 
 u s e   c o n s t a n t   S E R V E R _ S T A T U S _ I N _ T R A N S 	 	 = >   1 ; 
 u s e   c o n s t a n t   S E R V E R _ S T A T U S _ A U T O C O M M I T 	 	 = >   2 ; 
 u s e   c o n s t a n t   S E R V E R _ M O R E _ R E S U L T S _ E X I S T S 	 	 = >   8 ; 
 u s e   c o n s t a n t   S E R V E R _ Q U E R Y _ N O _ G O O D _ I N D E X _ U S E D 	 = >   1 6 ; 
 u s e   c o n s t a n t   S E R V E R _ Q U E R Y _ N O _ I N D E X _ U S E D 	 	 = >   3 2 ; 
 u s e   c o n s t a n t   S E R V E R _ S T A T U S _ C U R S O R _ E X I S T S 	 = >   6 4 ; 
 u s e   c o n s t a n t   S E R V E R _ S T A T U S _ L A S T _ R O W _ S E N T 	 = >   1 2 8 ; 
 u s e   c o n s t a n t   S E R V E R _ S T A T U S _ D B _ D R O P P E D 	 	 = >   2 5 6 ; 
 u s e   c o n s t a n t   S E R V E R _ S T A T U S _ N O _ B A C K S L A S H _ E S C A P E S   = >   5 1 2 ; 
 
 u s e   c o n s t a n t   C O M _ S L E E P 	 	 	 = >   0 ; 
 u s e   c o n s t a n t   C O M _ Q U I T 	 	 	 = >   1 ; 
 u s e   c o n s t a n t   C O M _ I N I T _ D B 	 	 = >   2 ; 
 u s e   c o n s t a n t   C O M _ Q U E R Y 	 	 	 = >   3 ; 
 u s e   c o n s t a n t   C O M _ F I E L D _ L I S T 	 	 = >   4 ; 
 u s e   c o n s t a n t   C O M _ C R E A T E _ D B 	 	 = >   5 ; 
 u s e   c o n s t a n t   C O M _ D R O P _ D B 	 	 = >   6 ; 
 u s e   c o n s t a n t   C O M _ R E F R E S H 	 	 = >   7 ; 
 u s e   c o n s t a n t   C O M _ S H U T D O W N 	 	 = >   8 ; 
 u s e   c o n s t a n t   C O M _ S T A T I S T I C S 	 	 = >   9 ; 
 u s e   c o n s t a n t   C O M _ P R O C E S S _ I N F O 	 	 = >   1 0 ; 
 u s e   c o n s t a n t   C O M _ C O N N E C T 	 	 = >   1 1 ; 
 u s e   c o n s t a n t   C O M _ P R O C E S S _ K I L L 	 	 = >   1 2 ; 
 u s e   c o n s t a n t   C O M _ D E B U G 	 	 	 = >   1 3 ; 
 u s e   c o n s t a n t   C O M _ P I N G 	 	 	 = >   1 4 ; 
 u s e   c o n s t a n t   C O M _ T I M E 	 	 	 = >   1 5 ; 
 u s e   c o n s t a n t   C O M _ D E L A Y E D _ I N S E R T 	 	 = >   1 6 ; 
 u s e   c o n s t a n t   C O M _ C H A N G E _ U S E R 	 	 = >   1 7 ; 
 u s e   c o n s t a n t   C O M _ B I N L O G _ D U M P 	 	 = >   1 8 ; 
 u s e   c o n s t a n t   C O M _ T A B L E _ D U M P 	 	 = >   1 9 ; 
 u s e   c o n s t a n t   C O M _ C O N N E C T _ O U T 	 	 = >   2 0 ; 
 u s e   c o n s t a n t   C O M _ R E G I S T E R _ S L A V E 	 	 = >   2 1 ; 
 u s e   c o n s t a n t   C O M _ S T M T _ P R E P A R E 	 	 = >   2 2 ; 
 u s e   c o n s t a n t   C O M _ S T M T _ E X E C U T E 	 	 = >   2 3 ; 
 u s e   c o n s t a n t   C O M _ S T M T _ S E N D _ L O N G _ D A T A 	 = >   2 4 ; 
 u s e   c o n s t a n t   C O M _ S T M T _ C L O S E 	 	 = >   2 5 ; 
 u s e   c o n s t a n t   C O M _ S T M T _ R E S E T 	 	 = >   2 6 ; 
 u s e   c o n s t a n t   C O M _ S E T _ O P T I O N 	 	 = >   2 7 ; 
 u s e   c o n s t a n t   C O M _ S T M T _ F E T C H 	 	 = >   2 8 ; 
 u s e   c o n s t a n t   C O M _ E N D 	 	 	 = >   2 9 ; 
 
 #   T h i s   i s   t a k e n   f r o m   i n c l u d e / m y s q l _ c o m . h 
 
 u s e   c o n s t a n t   M Y S Q L _ T Y P E _ D E C I M A L 	 	 = >   0 ; 
 u s e   c o n s t a n t   M Y S Q L _ T Y P E _ T I N Y 	 	 = >   1 ; 
 u s e   c o n s t a n t   M Y S Q L _ T Y P E _ S H O R T 	 	 = >   2 ; 
 u s e   c o n s t a n t   M Y S Q L _ T Y P E _ L O N G 	 	 = >   3 ; 
 u s e   c o n s t a n t   M Y S Q L _ T Y P E _ F L O A T 	 	 = >   4 ; 
 u s e   c o n s t a n t   M Y S Q L _ T Y P E _ D O U B L E 	 	 = >   5 ; 
 u s e   c o n s t a n t   M Y S Q L _ T Y P E _ N U L L 	 	 = >   6 ; 
 u s e   c o n s t a n t   M Y S Q L _ T Y P E _ T I M E S T A M P 	 = >   7 ; 
 u s e   c o n s t a n t   M Y S Q L _ T Y P E _ L O N G L O N G 	 = >   8 ; 
 u s e   c o n s t a n t   M Y S Q L _ T Y P E _ I N T 2 4 	 	 = >   9 ; 
 u s e   c o n s t a n t   M Y S Q L _ T Y P E _ D A T E 	 	 = >   1 0 ; 
 u s e   c o n s t a n t   M Y S Q L _ T Y P E _ T I M E 	 	 = >   1 1 ; 
 u s e   c o n s t a n t   M Y S Q L _ T Y P E _ D A T E T I M E 	 = >   1 2 ; 
 u s e   c o n s t a n t   M Y S Q L _ T Y P E _ Y E A R 	 	 = >   1 3 ; 
 u s e   c o n s t a n t   M Y S Q L _ T Y P E _ N E W D A T E 	 	 = >   1 4 ; 
 u s e   c o n s t a n t   M Y S Q L _ T Y P E _ V A R C H A R 	 	 = >   1 5 ; 
 u s e   c o n s t a n t   M Y S Q L _ T Y P E _ B I T 	 	 = >   1 6 ; 
 u s e   c o n s t a n t   M Y S Q L _ T Y P E _ N E W D E C I M A L 	 = >   2 4 6 ; 
 u s e   c o n s t a n t   M Y S Q L _ T Y P E _ E N U M 	 	 = >   2 4 7 ; 
 u s e   c o n s t a n t   M Y S Q L _ T Y P E _ S E T 	 	 = >   2 4 8 ; 
 u s e   c o n s t a n t   M Y S Q L _ T Y P E _ T I N Y _ B L O B 	 = >   2 4 9 ; 
 u s e   c o n s t a n t   M Y S Q L _ T Y P E _ M E D I U M _ B L O B 	 = >   2 5 0 ; 
 u s e   c o n s t a n t   M Y S Q L _ T Y P E _ L O N G _ B L O B 	 = >   2 5 1 ; 
 u s e   c o n s t a n t   M Y S Q L _ T Y P E _ B L O B 	 	 = >   2 5 2 ; 
 u s e   c o n s t a n t   M Y S Q L _ T Y P E _ V A R _ S T R I N G 	 = >   2 5 3 ; 
 u s e   c o n s t a n t   M Y S Q L _ T Y P E _ S T R I N G 	 	 = >   2 5 4 ; 
 u s e   c o n s t a n t   M Y S Q L _ T Y P E _ G E O M E T R Y 	 = >   2 5 5 ; 
 
 u s e   c o n s t a n t   N O T _ N U L L _ F L A G 	 	 = >   1 ; 
 u s e   c o n s t a n t   P R I _ K E Y _ F L A G 	 	 = >   2 ; 
 u s e   c o n s t a n t   U N I Q U E _ K E Y _ F L A G 	 	 = >   4 ; 
 u s e   c o n s t a n t   M U L T I P L E _ K E Y _ F L A G 	 	 = >   8 ; 
 u s e   c o n s t a n t   B L O B _ F L A G 	 	 	 = >   1 6 ; 
 u s e   c o n s t a n t   U N S I G N E D _ F L A G 	 	 = >   3 2 ; 
 u s e   c o n s t a n t   Z E R O F I L L _ F L A G 	 	 = >   6 4 ; 
 u s e   c o n s t a n t   B I N A R Y _ F L A G 	 	 = >   1 2 8 ; 
 u s e   c o n s t a n t   E N U M _ F L A G 	 	 	 = >   2 5 6 ; 
 u s e   c o n s t a n t   A U T O _ I N C R E M E N T _ F L A G 	 = >   5 1 2 ; 
 u s e   c o n s t a n t   T I M E S T A M P _ F L A G 	 	 = >   1 0 2 4 ; 
 u s e   c o n s t a n t   S E T _ F L A G 	 	 	 = >   2 0 4 8 ; 
 u s e   c o n s t a n t   N O _ D E F A U L T _ V A L U E _ F L A G 	 = >   4 0 9 6 ; 
 u s e   c o n s t a n t   N U M _ F L A G 	 	 	 = >   3 2 7 6 8 ; 
 
 # m y   $ M Y S E R V E R _ P A C K E T _ C O U N T ; 
 # m y   $ Q U E R Y _ H A N D L E R S ; 
 
 s u b   _ a u t h e n t i c a t e   { 
         m y   ( $ s e l f ,   $ d a t a )   =   @ _ ; 
         
         
         
         # p r i n t   "     ' " . $ d a t a . " '     \ n " ; 
         
         m y   $ d a t a b a s e ; 
 
         m y   $ p t r   =   0 ; 
         m y   $ h e a d e r _ f l a g s   =   s u b s t r ( $ d a t a ,   $ p t r ,   4 ) ; 
         $ p t r   =   $ p t r   +   4 ; 
         
         e v a l   { 
                 m y   $ c l i e n t _ f l a g s   =   s u b s t r ( $ d a t a ,   $ p t r ,   4 ) ; 
                 $ p t r   =   $ p t r   +   4 ; 
                 
                 m y   $ m a x _ p a c k e t _ s i z e   =   s u b s t r ( $ d a t a ,   $ p t r ,   4 ) ; 
                 $ p t r   =   $ p t r   +   4 ; 
                 
                 m y   $ c h a r s e t _ n u m b e r   =   s u b s t r ( $ d a t a ,   $ p t r ,   1 ) ; 
                 $ s e l f - > c h a r s e t ( o r d ( $ c h a r s e t _ n u m b e r ) ) ; 
                 $ p t r + + ; 
                 
                 m y   $ f i l l e r 1   =   s u b s t r ( $ d a t a ,   $ p t r ,   2 3 ) ; 
                 $ p t r   =   $ p t r   +   2 3 ; 
                 
                 m y   $ u s e r n a m e _ e n d   =   i n d e x ( $ d a t a ,   " \ 0 " ,   $ p t r ) ; 
                 m y   $ u s e r n a m e   =   s u b s t r ( $ d a t a ,   $ p t r ,   $ u s e r n a m e _ e n d   -   $ p t r ) ; 
                 $ p t r   =   $ u s e r n a m e _ e n d   +   1 ; 
                 
                 m y   $ s c r a m b l e _ b u f f ; 
                 
                 m y   $ s c r a m b l e _ l e n g t h   =   o r d ( s u b s t r ( $ d a t a ,   $ p t r ,   1 ) ) ; 
                 $ p t r + + ; 
                 
                 i f   ( $ s c r a m b l e _ l e n g t h   >   0 )   { 
                 $ s e l f - > s c r a m b l e ( s u b s t r ( $ d a t a ,   $ p t r ,   $ s c r a m b l e _ l e n g t h ) ) ; 
                 $ p t r   =   $ p t r   +   $ s c r a m b l e _ l e n g t h ; 
                 } 
                 
                 m y   $ d a t a b a s e _ e n d   =   i n d e x (   $ d a t a ,   " \ 0 " ,   $ p t r ) ; 
                 i f   ( $ d a t a b a s e _ e n d   ! =   - 1   )   { 
                   $ d a t a b a s e   =   s u b s t r ( $ d a t a ,   $ p t r ,   $ d a t a b a s e _ e n d   -   $ p t r ) ; 
                 } 
 #                 $ d a t a b a s e   | | =   ' N U L L ' ; 
               
   
                 $ s e l f - > d a t a b a s e ( $ d a t a b a s e ) ; 
                 $ s e l f - > u s e r n a m e ( $ u s e r n a m e ) ; 
 #                 $ s e l f - > p a s s w o r d ( $ d a t a b a s e ) ; 
 
 
                 p r i n t   $ u s e r n a m e ; 
                 p r i n t   '   ~ ~   ' . $ d a t a b a s e . " \ n " ; 
 
                 m y   $ m o d u l e   =   $ s e l f - > s e r v e r _ c l a s s ; 
 
                 u n l e s s   ( C l a s s : : I n s p e c t o r - > l o a d e d ( $ m o d u l e ) )   { 
                         r e q u i r e   C l a s s : : I n s p e c t o r - > f i l e n a m e ( $ m o d u l e ) ; 
                 } 
 
                 i f   ( m y   $ c o d e   =   $ m o d u l e - > c a n ( ' c h a n g e _ d b ' ) )   { 
                         e v a l   {   
                                 $ c o d e - > ( $ m o d u l e ,   $ s e l f ,   $ d a t a b a s e ) ; 
                         } ; 
 
                         i f   ( $ @ )   { 
                                 d i e   $ @ ; 
                         } 
 
                         u n l e s s   ( $ @ )   { 
                                 $ s e l f - > d a t a b a s e ( $ d a t a ) ; 
                         } 
                 } 
                 
         } ; 
 
         i f   ( $ @ )   { 
                 d i e   $ @ ; 
         } 
 
         i f   ( $ d a t a b a s e )   { 
                 $ s e l f - > a u t h e n t i c a t e d ( 1 ) ; 
                 $ s e l f - > s e n d _ o k ; 
         } 
         e l s e   { 
               $ s e l f - > s e n d _ e r r o r ( ' P l e a s e   c h o o s e   a   d a t a b a s e   ! ' ) ;   
         } 
 } 
 
 
 
 s u b   s e n d _ e r r o r   { 
 # # #   S e n d i n g   E r r o r   . . . 
 	 m y   ( $ s e l f ,   $ m e s s a g e ,   $ e r r n o ,   $ s q l s t a t e )   =   @ _ ; 
 # # # #   $ e r r n o 
 # # # #   $ s q l s t a t e 
 # # # #   $ m e s s a g e 
 	 #   h t t p : / / d e v . m y s q l . c o m / d o c / i n t e r n a l s / e n / e r r o r - p a c k e t . h t m l 
 
 	 $ m e s s a g e   =   ' U n k n o w n   M y S Q L   e r r o r '   i f   n o t   d e f i n e d   $ m e s s a g e ; 
 	 $ e r r n o   =   2 0 0 0   i f   n o t   d e f i n e d   $ e r r n o ; 
 	 $ s q l s t a t e   =   ' H Y 0 0 0 '   i f   n o t   d e f i n e d   $ s q l s t a t e ; 
 
 	 m y   $ p a y l o a d   =   c h r ( 0 x f f ) ; 	 	 #   f i e l d _ c o u n t ,   a l w a y s   =   0 x f f 
 	 $ p a y l o a d   . =   p a c k ( ' v ' ,   $ e r r n o ) ; 
 	 $ p a y l o a d   . =   ' # ' ; 	 	 	 	 #   s q l s t a t e   m a r k e r ,   a l w a y s   # 
 	 $ p a y l o a d   . =   $ s q l s t a t e ; 
 	 $ p a y l o a d   . =   $ m e s s a g e . " \ 0 " ; 
 
 
         $ s e l f - > w r i t e (   $ p a y l o a d ,   1 ) ; 
 
 } 
 
 
   
 # s u b   r e s o l v e _ q u e r y   { 
 #         m y   ( $ s e l f ,   $ q u e r y )   =   @ _ ; 
 # 
 #         $ s e l f - > S U P E R : : r e s o l v e _ q u e r y ( @ _ ) ; 
 # 
 #         m y   $ s y s _ q u e r y   =   $ s e l f - > r e s o l v e _ s y s _ q u e r y ( $ q u e r y ) ; 
 # 
 #         p r i n t   "   r e f   " . r e f ( $ s e l f ) . "   \ n " ; 
 # 
 # 
 #         p r i n t   D u m p e r ( \ @ I S A ) ; 
 #         r e t u r n   $ s y s _ q u e r y   i f   d e f i n e d   $ s y s _ q u e r y ; 
 # 
 #         p r i n t   " p a r e n t   r e s o l v e   q u e r y   \ n " ; 
 # 
 # 
 # #         i f   ( $ q u e r y   e q   ' s e l e c t   @ @ v e r s i o n _ c o m m e n t   l i m i t   1 ' )   { 
 # #                 r e t u r n   ' t e s t ' ; 
 # #         } 
 #         r e t u r n   ' t e s t _ t a b l e s ' ; 
 # } 
 
 
 s u b   r e s o l v e _ s y s _ q u e r y   { 
         m y   ( $ s e l f ,   $ q u e r y )   =   @ _ ; 
 
         i f   ( $ q u e r y   e q   ' s e l e c t   @ @ v e r s i o n _ c o m m e n t   l i m i t   1 ' )   { 
                 r e t u r n   ' t e s t ' ; 
         } 
         e l s i f   ( $ q u e r y   = ~   / a r r a y / )   { 
                 r e t u r n   ' t e s t _ m y a r r a y ' ; 
         } 
         e l s i f   ( $ q u e r y   = ~   / d a t a b a s e s / )   { 
                 r e t u r n   ' t e s t _ d b ' ; 
         } 
         e l s i f   ( $ q u e r y   e q   ' s h o w   t a b l e s ' )   { 
                 r e t u r n   ' s h o w _ t a b l e s ' ; 
         } 
         e l s i f   ( $ q u e r y   = ~   / ^ s e l e c t   d a t a b a s e / i )   { 
                 r e t u r n   ' s e l e c t _ d b ' ; 
         } 
         e l s i f   ( $ q u e r y   = ~   / ^ u s e /   o r   $ q u e r y   = ~   / ^ t e s t _ c h a n g e _ d b / i   o r   $ q u e r y   = ~   / ^ s e l e c t   d a t a b a s e / i )   { 
                 r e t u r n   ' c h a n g e _ d b ' ; 
         } 
 
         r e t u r n   u n d e f ; 
 } 
 
 
 s u b   i s a   { 
         m y   ( $ s e l f ,   $ m o d u l e )   =   @ _ ; 
 
 
         i f   ( $ m o d u l e )   { 
         p r i n t   " i s a   - >   $ m o d u l e   \ n " ; 
                 u n l e s s   ( C l a s s : : I n s p e c t o r - > l o a d e d ( $ m o d u l e ) )   { 
                         r e q u i r e   C l a s s : : I n s p e c t o r - > f i l e n a m e ( $ m o d u l e ) ; 
                 } 
                 p u s h   @ I S A ,   $ m o d u l e ; 
 
                 #   r e g i s t e r   s y s t e m   s t a t e s         
                 m y   @ m e t h o d s   =   C l a s s : : I n s p e c t o r - > m e t h o d s ( 
                         $ m o d u l e , 
                         ' e x p a n d e d ' , 
                         ' p u b l i c ' 
                 ) ; 
                 
                 f o r e a c h   m y   $ m e t h o d   ( @ {   $ m e t h o d s [ 0 ]   } )   { 
                         m y   ( $ f u l l ,   $ c l a s s ,   $ m e t h o d ,   u n d e f   )   =   @ {   $ m e t h o d   } ; 
                         i f   ( $ c l a s s   e q   $ m o d u l e )   { 
                                 $ s e l f - > s e s s i o n - > _ r e g i s t e r _ s t a t e ( $ m e t h o d ,   $ s e l f ) ; 
                         } 
                 } 
 
 
         } 
 
 u s e   D a t a : : D u m p e r ; 
 p r i n t   D u m p e r ( \ @ I S A ) ; 
 
         r e t u r n   @ I S A ; 
 } 
 
 s u b   h a n d l e _ c l i e n t _ i n p u t   { 
         m y   (   $ k e r n e l ,   $ s e s s i o n ,   $ h e a p ,   $ s e l f   )   =   @ _ [   K E R N E L ,   S E S S I O N ,   H E A P ,   O B J E C T ] ; 
         m y   $ d a t a   =   $ _ [ A R G 0 ] ; 
 
         i f   ( l e n g t h ( $ d a t a )   >   1 )   { 
                 $ s e l f - > p a c k e t _ c o u n t ( $ s e l f - > p a c k e t _ c o u n t   +   1 ) ; 
         } 
 	 r e t u r n   u n d e f   i f   l e n g t h ( $ d a t a )   < =   1 ; 
         
         u n l e s s   (   $ s e l f - > a u t h e n t i c a t e d )   { 
                 $ s e l f - > _ a u t h e n t i c a t e ( $ d a t a ) ; 
 	 } 
 	 e l s e   { 
                 m y   $ h e a d e r _ f l a g s   =   s u b s t r ( $ d a t a ,   0 ,   4 ) ; 
                 $ d a t a   =   s u b s t r ( $ d a t a ,   4 ) ; 
                 
 #                 m y   $ c o m m a n d       =   u n p a c k ( ' C ' ,   s u b s t r ( $ h e a d e r _ f l a g s ,   3 ,   1 ) ) ; 
 #                 m y   $ d a t a _             =   l e n g t h ( $ d a t a )   >   3   ?   s u b s t r ( $ d a t a ,   3 )   :   u n d e f ; 
 
                 m y   $ e v e n t ; 
 
                 $ e v e n t   =   $ s e l f - > r e s o l v e _ s y s _ q u e r y ( $ d a t a ) ; 
 
                 u n l e s s   ( $ e v e n t )   { 
 
                         e v a l   { 
                                 $ e v e n t   =   $ s e l f - > r e s o l v e _ q u e r y ( $ d a t a ) ; 
                         } ; 
 
                         i f   ( $ @ )   { 
                                 p r i n t   $ @ . " \ n " ; 
                                 $ e v e n t   =   $ s e l f - > r e s o l v e _ s y s _ q u e r y ( $ d a t a ) ; 
                         } 
                     
                         i f   ( $ s e l f - > w a n t _ d b _ n a m e )   { 
                 #                         p r i n t   " q u e r y   i s   D B   n a m e   ! !   \ n " ; 
                                 $ e v e n t   =   ' s e l e c t _ d b ' ; 
                         } 
                 } ; 
                 p r i n t   " f i r i n g   q u e r y   i s   $ d a t a   /   $ e v e n t   \ n " ; 
 
                 P O E : : K e r n e l - > p o s t ( 
                         $ s e l f - > s e s s i o n _ i d ,   
                         $ e v e n t ,   
                         $ d a t a 
                 ) ; 
       } 
 } 
 
 
 s u b   n e w   { 
         m y   $ c l a s s   =   s h i f t ; 
         m y   $ s e l f   =   $ c l a s s - > S U P E R : : n e w ( @ _ ) ; 
 
         r e t u r n   $ s e l f ; 
 } 
 
 
 s u b   _ s e n d _ d e f i n i t i o n s   { 
         m y   ( $ s e l f ,   $ d e f i n i t i o n s ,   $ s k i p _ e n v e l o p e )   =   @ _ ; 
 
 	 i f   ( n o t   d e f i n e d   $ s k i p _ e n v e l o p e )   { 
                 $ s e l f - > w r i t e ( _ l e n g t h C o d e d B i n a r y ( ( s c a l a r ( @ { $ d e f i n i t i o n s } ) ) ) ) ; 
 	 } 
 
 	 m y   $ l a s t _ s e n d _ r e s u l t ; 
 
 	 f o r e a c h   m y   $ d e f i n i t i o n   ( @ { $ d e f i n i t i o n s } )   { 
                 $ d e f i n i t i o n   =   n e w _ d e f i n i t i o n ( n a m e   = >   $ d e f i n i t i o n )   u n l e s s   r e f ( $ d e f i n i t i o n )   e q   ' D B I x : : M y S e r v e r : : D e f i n i t i o n ' ; 
                 $ s e l f - > s e n d _ d e f i n i t i o n ( $ d e f i n i t i o n ) ; 
 	 } ; 
 
         i f   ( n o t   d e f i n e d   $ s k i p _ e n v e l o p e )   { 
                 $ s e l f - > s e n d _ d e f i n i t i o n s _ e o f ( ) ; 
 	 }   
         e l s e   { 
 	 	 r e t u r n   $ l a s t _ s e n d _ r e s u l t ; 
 	 } 
 } 
 
 
 s u b   s e n d _ h e a d e r   { 
         m y   ( $ s e l f ,   $ f i e l d _ c o u n t )   =   @ _ ; 
 
         $ s e l f - > w r i t e ( _ l e n g t h C o d e d B i n a r y ( $ f i e l d _ c o u n t ) ) ; 
 } 
 
 s u b   s e n d _ d e f i n i t i o n s _ e o f   { 
         m y   ( $ s e l f )   =   @ _ ; 
         $ s e l f - > w r i t e ( c h r ( 0 x f e ) ) ; 
 } 
 
 
 s u b   s e n d _ d e f i n i t i o n   { 
         m y   ( $ s e l f ,   $ d e f i n i t i o n )   =   @ _ ; 
 	 
 	 m y   ( 
 	 	 $ f i e l d _ c a t a l o g ,   $ f i e l d _ d b ,   $ f i e l d _ t a b l e , 
 	 	 $ f i e l d _ o r g _ t a b l e ,   $ f i e l d _ n a m e ,   $ f i e l d _ o r g _ n a m e , 
 	 	 $ f i e l d _ l e n g t h ,   $ f i e l d _ t y p e ,   $ f i e l d _ f l a g s , 
 	 	 $ f i e l d _ d e c i m a l s ,   $ f i e l d _ d e f a u l t 
 	 )   =   (   
 	 	 $ d e f i n i t i o n - > [ F I E L D _ C A T A L O G ] ,   $ d e f i n i t i o n - > [ F I E L D _ D B ] ,   
 	 	 $ d e f i n i t i o n - > [ F I E L D _ T A B L E ] ,   $ d e f i n i t i o n - > [ F I E L D _ O R G _ T A B L E ] , 
 	 	 $ d e f i n i t i o n - > [ F I E L D _ N A M E ] ,   $ d e f i n i t i o n - > [ F I E L D _ O R G _ N A M E ] , 
 	 	 $ d e f i n i t i o n - > [ F I E L D _ L E N G T H ] ,   $ d e f i n i t i o n - > [ F I E L D _ T Y P E ] , 
 	 	 $ d e f i n i t i o n - > [ F I E L D _ F L A G S ] ,   $ d e f i n i t i o n - > [ F I E L D _ D E C I M A L S ] , 
 	 	 $ d e f i n i t i o n - > [ F I E L D _ D E F A U L T ] 
 	 ) ; 
 
 	 m y   $ p a y l o a d   =   j o i n ( ' ' ,   m a p   {   _ l e n g t h C o d e d S t r i n g ( $ _ )   }   ( 
 	 	 $ f i e l d _ c a t a l o g ,   $ f i e l d _ d b ,   $ f i e l d _ t a b l e , 
 	 	 $ f i e l d _ o r g _ t a b l e ,   $ f i e l d _ n a m e ,   $ f i e l d _ o r g _ n a m e 
 	 ) ) ; 
 
 	 $ p a y l o a d   . =   c h r ( 0 x 0 c ) ; 	 #   F i l l e r 
 	 $ p a y l o a d   . =   p a c k ( ' v ' ,   1 1 ) ; 	 	 #   U S   A S C I I 
 	 $ p a y l o a d   . =   p a c k ( ' V ' ,   $ f i e l d _ l e n g t h ) ; 
 	 $ p a y l o a d   . =   c h r ( $ f i e l d _ t y p e ) ; 
 	 $ p a y l o a d   . =   d e f i n e d   $ f i e l d _ f l a g s   ?   p a c k ( ' v ' ,   $ f i e l d _ f l a g s )   :   p a c k ( ' v ' ,   0 ) ; 
 	 $ p a y l o a d   . =   d e f i n e d   $ f i e l d _ d e c i m a l s   ?   c h r ( $ f i e l d _ d e c i m a l s )   :   p a c k ( ' v ' , ' 0 ' ) ; 
 	 $ p a y l o a d   . =   p a c k ( ' v ' ,   0 ) ; 	 	 #   F i l l e r 
 	 $ p a y l o a d   . =   _ l e n g t h C o d e d S t r i n g ( $ f i e l d _ d e f a u l t ) ; 
 
       $ s e l f - > w r i t e ( $ p a y l o a d ) ; 
 } 
 
 
 s u b   s e n d _ e o f   { 
         m y   ( $ s e l f ,   $ w a r n i n g _ c o u n t ,   $ s e r v e r _ s t a t u s )   =   @ _ ; 
 
 	 m y   $ p a y l o a d ; 
 
 	 $ w a r n i n g _ c o u n t   =   0   i f   n o t   d e f i n e d   $ w a r n i n g _ c o u n t ; 
 	 $ s e r v e r _ s t a t u s   =   S E R V E R _ S T A T U S _ A U T O C O M M I T   i f   n o t   d e f i n e d   $ s e r v e r _ s t a t u s ; 
 
 	 $ p a y l o a d   . =   c h r ( 0 x f e ) ; 
 	 $ p a y l o a d   . =   p a c k ( ' v ' ,   $ w a r n i n g _ c o u n t ) ; 
 	 $ p a y l o a d   . =   p a c k ( ' v ' ,   $ s e r v e r _ s t a t u s ) ; 
 
       $ s e l f - > w r i t e ( $ p a y l o a d ,   1 ) ; 
 } 
 
 s u b   s e n d _ o k   { 
         m y   ( $ s e l f ,   $ m e s s a g e ,   $ a f f e c t e d _ r o w s ,   $ i n s e r t _ i d ,   $ p r i n t i n g _ c o u n t )   =   @ _ ; 
         
         m y   $ d a t a ; 
         
         $ a f f e c t e d _ r o w s   =   0   i f   n o t   d e f i n e d   $ a f f e c t e d _ r o w s ; 
         $ p r i n t i n g _ c o u n t   =   0   i f   n o t   d e f i n e d   $ p r i n t i n g _ c o u n t ; 
         
         $ d a t a   . =   " \ 0 " ; 
         $ d a t a   . =   _ l e n g t h C o d e d B i n a r y ( $ a f f e c t e d _ r o w s ) ; 
         $ d a t a   . =   _ l e n g t h C o d e d B i n a r y ( $ i n s e r t _ i d ) ; 
         $ d a t a   . =   p a c k ( ' v ' ,   S E R V E R _ S T A T U S _ A U T O C O M M I T ) ; 
         $ d a t a   . =   p a c k ( ' v ' ,   $ p r i n t i n g _ c o u n t ) ; 
         $ d a t a   . =   _ l e n g t h C o d e d S t r i n g ( $ m e s s a g e ) ; 
         
         $ s e l f - > w r i t e (   $ d a t a ,   1 ) ; 
 } 
 
 s u b   w r i t e   { 
         m y   ( $ s e l f ,   $ m e s s a g e ,   $ r e i n i t )   =   @ _ ; 
 
         m y   $ h e a d e r ; 
         $ h e a d e r   . =   s u b s t r ( p a c k ( ' V ' , l e n g t h ( $ m e s s a g e ) ) , 0 , 3 ) ; 
         $ h e a d e r   . =   c h r ( $ s e l f - > p a c k e t _ c o u n t   %   2 5 6 ) ; 
       
         i f   ( $ r e i n i t )   { 
                 $ s e l f - > p a c k e t _ c o u n t ( 0 ) ; 
         } 
         e l s e   {   
                 $ s e l f - > p a c k e t _ c o u n t ( $ s e l f - > p a c k e t _ c o u n t   +   1 ) ; 
         } 
 
         $ s e l f - > w h e e l - > p u t ( $ h e a d e r . $ m e s s a g e ) ; 
 } 
 
 
 s u b   _ l e n g t h C o d e d S t r i n g   { 
 	 m y   ( $ s t r i n g )   =   @ _ ; 
 	 r e t u r n   c h r ( 0 )   i f   ( n o t   d e f i n e d   $ s t r i n g ) ; 
 	 r e t u r n   c h r ( 2 5 3 ) . s u b s t r ( p a c k ( ' V ' , l e n g t h ( $ s t r i n g ) ) , 0 , 3 ) . $ s t r i n g ; 
 } 
 
 s u b   _ l e n g t h C o d e d B i n a r y   { 
 	 m y   ( $ n u m b e r )   =   @ _ ; 
 	 i f   ( n o t   d e f i n e d   $ n u m b e r )   { 
 	 	 r e t u r n   c h r ( 2 5 1 ) ; 
 	 } 
         e l s i f   ( $ n u m b e r   <   2 5 1 )   { 
 	 	 r e t u r n   c h r ( $ n u m b e r ) ; 
 	 } 
         e l s i f   ( $ n u m b e r   <   0 x 1 0 0 0 0 )   { 
 	 	 r e t u r n   c h r ( 2 5 2 ) . p a c k ( ' v ' ,   $ n u m b e r ) ; 
 	 } 
         e l s i f   ( $ n u m b e r   <   0 x 1 0 0 0 0 0 0 )   { 
 	 	 r e t u r n   c h r ( 2 5 3 ) . s u b s t r ( p a c k ( ' V ' ,   $ n u m b e r ) ,   0 ,   3 ) ; 
 	 } 
         e l s e   { 
 	 	 r e t u r n   c h r ( 2 5 4 ) . p a c k ( ' V ' ,   $ n u m b e r   > >   3 2 ) . p a c k ( ' V ' ,   $ n u m b e r   &   0 x f f f f f f f f ) ; 
 	 } 
 } 
 
 s u b   h a n d l e _ c l i e n t _ e r r o r   { 
         m y   ( $ s e l f )   =   s h i f t ; 
         m y   ( $ k e r n e l ,   $ s e s s i o n ,   $ h e a p   )   =   @ _ [   K E R N E L ,   S E S S I O N ,   H E A P ] ; 
         m y   ( $ o p e r a t i o n ,   $ e r r n u m ,   $ e r r s t r ,   $ w h e e l _ i d )   =   @ _ [ A R G 0 . . A R G 3 ] ; 
         p r i n t   " W h e e l   $ w h e e l _ i d   g e n e r a t e d   $ o p e r a t i o n   e r r o r   $ e r r n u m :   $ e r r s t r \ n " ; 
 } 
 
 s u b   h a n d l e _ c l i e n t _ c o n n e c t   { 
         m y   ( $ s e l f )   =   s h i f t ; 
 
         $ s e l f - > b a n n e r ( " P O E : : C o m p o n e n t : : D B I x : : M y S e r v e r   " . $ V E R S I O N . " \ 0 " ) ; 
         $ s e l f - > s a l t ( j o i n ( ' ' , m a p   {   c h r ( i n t ( r a n d ( 2 5 5 ) ) )   }   ( 1 . . 2 0 ) ) ) ;       
         $ s e l f - > c h a r s e t ( 0 x 2 1 ) ; 
         $ s e l f - > t i d ( $ $ ) ; 
         $ s e l f - > p a c k e t _ c o u n t ( 0 ) ; 
 
         m y   $ p a y l o a d   =   c h r ( 1 0 ) ; 
         $ p a y l o a d   . =   $ s e l f - > b a n n e r ; 
         $ p a y l o a d   . =   p a c k ( ' V ' ,   $ s e l f - > t i d ) ; 
         $ p a y l o a d   . =   s u b s t r ( $ s e l f - > s a l t , 0 , 8 ) . " \ 0 " ; 
 #         $ p a y l o a d   . =   p a c k ( ' v ' ,   C L I E N T _ L O N G _ P A S S W O R D   |   C L I E N T _ C O N N E C T _ W I T H _ D B   |   C L I E N T _ P R O T O C O L _ 4 1   |   C L I E N T _ S E C U R E _ C O N N E C T I O N ) ; 
         $ p a y l o a d   . =   p a c k ( ' v ' ,   C L I E N T _ C O N N E C T _ W I T H _ D B   |   C L I E N T _ P R O T O C O L _ 4 1   |   C L I E N T _ S E C U R E _ C O N N E C T I O N ) ; 
         $ p a y l o a d   . =   $ s e l f - > c h a r s e t ; 
         $ p a y l o a d   . =   p a c k ( ' v ' ,   S E R V E R _ S T A T U S _ A U T O C O M M I T ) ; 
         $ p a y l o a d   . =   " \ 0 "   x   1 3 ; 
         $ p a y l o a d   . =   s u b s t r ( $ s e l f - > s a l t , 8 ) . " \ 0 " ; 
         
         $ s e l f - > w r i t e ( $ p a y l o a d ) ; 
 } 
 
 s u b   h a n d l e _ c l i e n t _ d i s c o n n e c t   { 
       m y   (   $ k e r n e l ,   $ s e s s i o n ,   $ h e a p   )   =   @ _ [   K E R N E L ,   S E S S I O N ,   H E A P ] ; 
       
       p r i n t   " h a n d l e _ c l i e n t _ d i s c o n n e c t " . " \ n "   i f   D E B U G ; 
 } 
 
 s u b   _ s e n d _ r o w s   { 
         m y   ( $ s e l f ,   $ r o w s )   =   @ _ ; 
         $ s e l f - > s e n d _ e o f   i f   n o t   d e f i n e d   $ r o w s ; 
         
 	 f o r e a c h   m y   $ r o w   ( @ { $ r o w s } )   { 
                 
                 m y   $ s m a l l _ d a t a ; 
                 f o r e a c h   ( @ { $ r o w } )   { 
                         i f   ( n o t   d e f i n e d   $ _ )   { 
                                 $ s m a l l _ d a t a   . =   c h r ( 2 5 1 ) ; 
                         } 
                         e l s e   { 
                                 $ s m a l l _ d a t a   . =   _ l e n g t h C o d e d S t r i n g ( $ _ ) ; 
                         } 
                 } 
                 
                 i f   ( d e f i n e d   $ s m a l l _ d a t a )   { 
                         m y   $ h e a d e r ; 
                         $ h e a d e r   . =   s u b s t r ( p a c k ( ' V ' , l e n g t h ( $ s m a l l _ d a t a ) ) , 0 , 3 ) ; 
                         $ h e a d e r   . =   c h r ( $ s e l f - > p a c k e t _ c o u n t   %   2 5 6 ) ; 
                         $ s e l f - > p a c k e t _ c o u n t ( $ s e l f - > p a c k e t _ c o u n t   +   1 ) ; 
                         
                         $ s e l f - > w h e e l - > p u t ( $ h e a d e r . $ s m a l l _ d a t a ) ; 
                 } 
 	 } 
 
         $ s e l f - > s e n d _ e o f ; 
 } 
 
 s u b   s e n d _ r e s u l t s   { 
         m y   ( $ s e l f ,   $ d e f i n i t i o n s ,   $ d a t a )   =   @ _ ; 
         $ s e l f - > _ s e n d _ d e f i n i t i o n s ( $ d e f i n i t i o n s ) ; 
         $ s e l f - > _ s e n d _ r o w s ( $ d a t a ) ; 
 } 
 
 s u b   t e s t   { 
         m y   (   $ k e r n e l ,   $ s e s s i o n ,   $ h e a p ,   $ s e l f   )   =   @ _ [   K E R N E L ,   S E S S I O N ,   H E A P ,   O B J E C T ] ; 
         m y   $ d a t a   =   $ _ [ A R G 0 ] ; 
 
         p r i n t   ' t e s t   ~   s e s s i o n   I D   =   ' . $ s e s s i o n - > I D . " \ n " ; 
       
         p r i n t   " v e r s i o n _ c o m m e n t   " . $ s e l f . "   ! !   \ n " ;   
         
         $ s e l f - > s e n d _ r e s u l t s ( [ n e w _ d e f i n i t i o n (   n a m e   = >   ' f i e l d '   ) ] ,   [ [ ' e c h o . p l   ~   ' . $ s e s s i o n - > I D ] ] ) ; 
 
         p r i n t   " d a   s h i t   w a s   s e n t   . . .   \ n " ; 
 } 
 
 s u b   t e s t _ m y a r r a y   { 
         m y   (   $ k e r n e l ,   $ s e s s i o n ,   $ h e a p ,   $ s e l f   )   =   @ _ [   K E R N E L ,   S E S S I O N ,   H E A P ,   O B J E C T ] ; 
         m y   $ d a t a   =   $ _ [ A R G 0 ] ; 
 
         p r i n t   ' t e s t   ~   s e s s i o n   I D   =   ' . $ s e s s i o n - > I D . " \ n " ; 
       
         p r i n t   " v e r s i o n _ c o m m e n t   ! !   \ n " ; 
 
         m y   @ s s ; 
 
         f o r   ( 0 . . 9 9 )   { 
                 $ s s [ $ _ ]   =   ( ) ; 
                 p u s h   @ {   $ s s [ $ _ ]   } ,   ' i n d e x   ' . $ _ ; 
         } 
             
         $ s e l f - > s e n d _ r e s u l t s ( [ n e w _ d e f i n i t i o n (   n a m e   = >   ' f i e l d '   ) ] ,   \ @ s s ) ; 
 
         p r i n t   " d a   D B   w a s   s e n t   . . .   \ n " ; 
 } 
 
 s u b   t e s t _ t a b l e s   { 
         m y   (   $ k e r n e l ,   $ s e s s i o n ,   $ h e a p ,   $ s e l f   )   =   @ _ [   K E R N E L ,   S E S S I O N ,   H E A P ,   O B J E C T ] ; 
         m y   $ d a t a   =   $ _ [ A R G 0 ] ; 
 
         p r i n t   ' t e s t   ~   s e s s i o n   I D   =   ' . $ s e s s i o n - > I D . " \ n " ; 
       
       	     p r i n t   " v e r s i o n _ c o m m e n t   ! !   \ n " ; 
       	 
             $ s e l f - > s e n d _ r e s u l t s ( [ n e w _ d e f i n i t i o n (   n a m e   = >   ' T a b l e s _ i n _ t e s t d b '   ) ] , 
                         [ [ ' m y t a b l e _ ' . $ s e s s i o n - > I D , ' m y t a b l e _ ' . $ s e s s i o n - > I D , ' m y t a b l e _ ' . $ s e s s i o n - > I D ] ] ) ;       	 
 
             p r i n t   " d a   D B   w a s   s e n t   . . .   \ n " ; 
 
 
 } 
 
 
 s u b   s h o w _ t a b l e s   { 
         m y   (   $ k e r n e l ,   $ s e s s i o n ,   $ h e a p ,   $ s e l f   )   =   @ _ [   K E R N E L ,   S E S S I O N ,   H E A P ,   O B J E C T ] ; 
         m y   $ d a t a   =   $ _ [ A R G 0 ] ; 
 
         $ s e l f - > s e n d _ r e s u l t s ( [ ' T a b l e s _ i n _ t e s t d b ' ] , 
                 [ ] ) ;       	 
 
 } 
 
 # s u b   c h a n g e _ d b   { 
 #         m y   (   $ k e r n e l ,   $ s e s s i o n ,   $ h e a p ,   $ s e l f   )   =   @ _ [   K E R N E L ,   S E S S I O N ,   H E A P ,   O B J E C T ] ; 
 #         m y   $ d a t a   =   $ _ [ A R G 0 ] ; 
 # 
 #         #   
 #         #   m o d u l e   l o a d   a n d   s h i t   . . . 
 #         # 
 #         m y   $ m o d u l e   =   $ s e l f - > s e r v e r _ c l a s s ; 
 # #         p r i n t   ( $ p a c k a g e . " - > c h a n g e _ d b ( " . $ d a t a . " ) " ) ; 
 # 
 #         u n l e s s   ( C l a s s : : I n s p e c t o r - > l o a d e d ( $ m o d u l e ) )   { 
 #                 r e q u i r e   C l a s s : : I n s p e c t o r - > f i l e n a m e ( $ m o d u l e ) ; 
 #         } 
 # 
 #         i f   ( m y   $ c o d e   =   $ m o d u l e - > c a n ( ' c h a n g e _ d b ' ) )   { 
 #                 $ c o d e - > ( $ m o d u l e ,   $ s e l f ,   $ d a t a ) ;   
 #         } 
 # 
 # # 
 # #                 P O E : : K e r n e l - > p o s t ( 
 # #                         $ s e l f - > s e s s i o n _ i d ,   
 # #                         ' a _ d u m m y _ q u e r y ' ,   
 # #                         $ d a t a 
 # #                 ) ; 
 # 
 #         
 # 
 # #         e v a l   ( $ p a c k a g e . " - > c h a n g e _ d b ( " . $ d a t a . " ) " ) ; 
 # 
 #         p r i n t   ' c h a n g e   d b   ' . $ d a t a . "   o k   ! ! \ n " ; 
 # 
 # } 
 
 s u b   s e l e c t _ d b   { 
         m y   (   $ k e r n e l ,   $ s e s s i o n ,   $ h e a p ,   $ s e l f   )   =   @ _ [   K E R N E L ,   S E S S I O N ,   H E A P ,   O B J E C T ] ; 
         m y   $ d a t a   =   $ _ [ A R G 0 ] ; 
 
         p r i n t   ' t e s t   ~   s e s s i o n   I D   =   ' . $ s e s s i o n - > I D . " \ n " ; 
       
 
         #   
         #   m o d u l e   l o a d   a n d   s h i t   . . . 
         # 
         m y   $ m o d u l e   =   $ s e l f - > s e r v e r _ c l a s s ; 
 #         p r i n t   ( $ p a c k a g e . " - > c h a n g e _ d b ( " . $ d a t a . " ) " ) ; 
 
       	     p r i n t   " s e l e c t   D B   $ d a t a   \ n " ; 
       	 
         i f   ( $ d a t a   = ~   / \ ( \ ) / )   { 
                 $ s e l f - > w a n t _ d b _ n a m e ( 1 ) ; 
                 p r i n t   " I   w a n t   t h e   n a m e   o f   t h e   d b   \ n " ; 
                 $ d a t a   =   ' t e s t d b ' ; 
 
 #                 $ s e l f - > s e n d _ r e s u l t s ( [ ' D a t a b a s e ' ] ,   [ [ ' t e s t d b ' ] ] ) ;   
                 $ s e l f - > s e n d _ r e s u l t s ( [ ' d a t a b a s e ( ) ' ] ,   [ [ $ s e l f - > d a t a b a s e ] ] ) ;   
 
         } 
         e l s e   { 
 
                 $ s e l f - > w a n t _ d b _ n a m e ( 0 ) ; 
 
                 u n l e s s   ( C l a s s : : I n s p e c t o r - > l o a d e d ( $ m o d u l e ) )   { 
                         r e q u i r e   C l a s s : : I n s p e c t o r - > f i l e n a m e ( $ m o d u l e ) ; 
                 } 
 
                 i f   ( m y   $ c o d e   =   $ m o d u l e - > c a n ( ' c h a n g e _ d b ' ) )   { 
                         e v a l   {   
                                 $ c o d e - > ( $ m o d u l e ,   $ s e l f ,   $ d a t a ) ; 
                         } ; 
 
                         u n l e s s   ( $ @ )   { 
                                 $ s e l f - > d a t a b a s e ( $ d a t a ) ; 
                         } 
                 } 
 
                 
 
     p r i n t   " d a   " . $ d a t a . "   D B   w a s   s e n t   . . .   \ n " ; 
                 $ s e l f - > s e n d _ r e s u l t s ( [ n e w _ d e f i n i t i o n (   n a m e   = >   ' D a t a b a s e '   ) ] ,   [ [ $ d a t a ] ] ) ;   	 
 
         } 
 
 
 #         $ d a t a   | | =   ' t e s t d b ' ; 
 
 
 #             $ s e l f - > s e n d _ r e s u l t s ( [ n e w _ d e f i n i t i o n (   n a m e   = >   ' D a t a b a s e '   ) ] , 
 #                 [ [ ' t e s t d b ' ] , [ ' m y d b ' ] , [ ' D a t a b a s e T e s t ' ] , [ ' t e s t d b ' ] ] ) ;   	 
 
     
 
 #         $ s e l f - > s e n d _ r e s u l t s ( [ n e w _ d e f i n i t i o n (   n a m e   = >   ' D a t a b a s e '   ) ] , [ [ ' t e s t d b ' ] ] ) ;   	 
 
 
         
 
 } 
 s u b   t e s t _ d b   { 
         m y   (   $ k e r n e l ,   $ s e s s i o n ,   $ h e a p ,   $ s e l f   )   =   @ _ [   K E R N E L ,   S E S S I O N ,   H E A P ,   O B J E C T ] ; 
         m y   $ d a t a   =   $ _ [ A R G 0 ] ; 
 
       
         p r i n t   " s h o w   d a t a b a s e s   ! !   \ n " ; 
       	 
         $ s e l f - > s e n d _ r e s u l t s ( [ n e w _ d e f i n i t i o n (   n a m e   = >   ' D a t a b a s e '   ) ] , 
                 [ [ ' m y d b ' ] , [ ' D a t a b a s e T e s t ' ] , [ ' t e s t d b ' ] ] ) ;   	 
 
 } 
 
 s u b   n e w _ d e f i n i t i o n   { 
 	 m y   % p a r a m s   =   @ _ ; 
 
 	 m y   $ d e f i n i t i o n   =   b l e s s ( [ ] ,   ' D B I x : : M y S e r v e r : : D e f i n i t i o n ' ) ; 
 	 $ d e f i n i t i o n - > [ F I E L D _ C A T A L O G ]   =   $ p a r a m s { c a t a l o g } ; 
 	 $ d e f i n i t i o n - > [ F I E L D _ D B ]   =   $ p a r a m s { d b }   ?   $ p a r a m s { d b }   :   $ p a r a m s { d a t a b a s e } ; 
 	 $ d e f i n i t i o n - > [ F I E L D _ T A B L E ]   =   $ p a r a m s { t a b l e } ; 
 	 $ d e f i n i t i o n - > [ F I E L D _ O R G _ T A B L E ]   =   $ p a r a m s { o r g _ t a b l e } ; 
 	 $ d e f i n i t i o n - > [ F I E L D _ N A M E ]   =   $ p a r a m s { n a m e } ; 
 	 $ d e f i n i t i o n - > [ F I E L D _ O R G _ N A M E ]   =   $ p a r a m s { o r g _ n a m e } ; 
 	 $ d e f i n i t i o n - > [ F I E L D _ L E N G T H ]   =   d e f i n e d   $ p a r a m s { l e n g t h }   ?   $ p a r a m s { l e n g t h }   :   0 ; 
 	 $ d e f i n i t i o n - > [ F I E L D _ T Y P E ]   =   d e f i n e d   $ p a r a m s { t y p e }   ?   $ p a r a m s { t y p e }   :   M Y S Q L _ T Y P E _ S T R I N G ; 
 	 $ d e f i n i t i o n - > [ F I E L D _ F L A G S ]   =   d e f i n e d   $ p a r a m s { f l a g s }   ?   $ p a r a m s { f l a g s }   :   0 ; 
 	 $ d e f i n i t i o n - > [ F I E L D _ D E C I M A L S ]   =   $ p a r a m s { d e c i m a l s } ; 
 	 $ d e f i n i t i o n - > [ F I E L D _ D E F A U L T ]   =   $ p a r a m s { d e f a u l t } ; 
 	 r e t u r n   $ d e f i n i t i o n ; 
 } 
 
 
 
 
 
 
 1 ; 
 
 
 
 